@@ -5,46 +5,50 @@ package App::ccdiff;
 use 5.014000;
 use warnings;
 use charnames ();
+use File::Find;
 use List::Util      qw( first  max     );
 use Term::ANSIColor qw(:constants color);
 use Getopt::Long    qw(:config bundling);
 
-our $VERSION = "0.31";
+our $VERSION = "0.33";
 our $CMD     = $0 =~ s{.*/}{}r;
 
 sub usage {
     my $err = shift and select STDERR;
     say "usage: $CMD [options] file1 [file2]";
+    say "       $CMD [options] dir1   dir2";
     say "       $CMD --man | --info";
     say "	file1 or file2 can be - (but not both)";
-    say "   -V    --version      Show version and exit";
-    say "   -v[1] --verbose[=1]  Set verbosity";
+    say "   -V     --version      Show version and exit";
+    say "   -v[1]  --verbose[=1]  Set verbosity";
     say "  Diff options:";
-    say "   -U    --utf-8                 Input is in UTF-8";
-    say "   -u[3] --unified=3             Show a unified diff";
-    say "         --no-header             Skip header with file names/stamps";
-    say "   -I    --index                 Add indices to the change chunks";
-    say "   -I n  --index=4               Only show chunk n";
-    say "   -w    --ignore-all-space      Ignore all whitespace";
-    say "   -b    --ignore-space-change   Ignore horizontal whitespace changes";
-    say "   -Z    --ignore-trailing-space Ignore whitespace at line ending";
-    say "   -B    --ignore-blank-lines    Ignore changes where lines are all blank";
-    say "   -i    --ignore-case           Ignore case changes";
+    say "   -U     --utf-8                 Input is in UTF-8";
+    say "   -u[3]  --unified=3             Show a unified diff";
+    say "          --no-header             Skip header with file names/stamps";
+    say "   -I     --index                 Add indices to the change chunks";
+    say "   -I n   --index=4               Only show chunk n";
+    say "   -w     --ignore-all-space      Ignore all whitespace";
+    say "   -b     --ignore-space-change   Ignore horizontal whitespace changes";
+    say "   -Z     --ignore-trailing-space Ignore whitespace at line ending";
+    say "   -B     --ignore-blank-lines    Ignore changes where lines are all blank";
+    say "   -i     --ignore-case           Ignore case changes";
+    say "   --dc=C --diff-class=C          Select Algorithm::Diff (AD or PP)";
+    say "                                      or Algorithm::Diff::XS (XS = default)";
     say "  Other options:";
-    say "   -t n  --threshold=2  Horizontal line diff threshold";
-    say "   -h n  --heuristics=n Horizontal char diff threshold";
-    say "   -e n  --ellipsis=n   Compress horizontal equal sections";
-    say "   -m    --markers      Use markers to indicate change positions";
-    say "   -a    --ascii        Use ASCII instead of Unicode indicators";
-    say "         --no-color     Reset all colors to none.";
-    say "         --list-colors  List available colors and exit";
-    say "         --old=red      Color to indicate removed content";
-    say "         --new=green    Color to indicate added   content";
-    say "         --bg=white     Background color for colored indicators";
-    say "   -p    --pink         Shortcut for --old=magenta";
-    say "   -r    --reverse      Reverse/invert the colors of the indicators";
-    say "   -s    --swap         Swap old/new color indicators";
-    say "         --settings     Show default settings (after reading rc)";
+    say "   -t n   --threshold=2  Horizontal line diff threshold";
+    say "   -h n   --heuristics=n Horizontal char diff threshold";
+    say "   -e n   --ellipsis=n   Compress horizontal equal sections";
+    say "   -m     --markers      Use markers to indicate change positions";
+    say "   -a     --ascii        Use ASCII instead of Unicode indicators";
+    say "          --no-color     Reset all colors to none.";
+    say "          --list-colors  List available colors and exit";
+    say "          --old=red      Color to indicate removed content";
+    say "          --new=green    Color to indicate added   content";
+    say "          --bg=white     Background color for colored indicators";
+    say "   -p     --pink         Shortcut for --old=magenta";
+    say "   -r     --reverse      Reverse/invert the colors of the indicators";
+    say "   -s     --swap         Swap old/new color indicators";
+    say "          --settings     Show default settings (after reading rc)";
     exit $err;
     } # usage
 
@@ -100,6 +104,7 @@ my $rev_color = $rc{bg};
 my $cli_color = $ENV{CLICOLOR};	# https://bixense.com/clicolors/
 my $no_colors = $ENV{NO_COLOR}; # https://no-color.org
 my $list_colors;
+my $diff_class;
 
 if ($no_colors) {
     $ENV{CLICOLOR_FORCE}	and $no_colors = 0;
@@ -118,6 +123,7 @@ unless (caller) {
 	  "info"	=> sub { pod_text  (); },
 
 	"U|utf-8!"		=> \$opt_U,
+	  "dc|diff-class=s"	=> \$diff_class,
 
     #   "c|context:3"		=> \$opt_c,	# implement context-diff?
 	"u|unified:3"		=> \$opt_u,
@@ -252,11 +258,59 @@ sub set_options {
 	}} : undef;
     } # set_options
 
-my $diff_class = eval { require Algorithm::Diff::XS; "Algorithm::Diff::XS"; }
-	      || eval { require Algorithm::Diff;     "Algorithm::Diff";     }
-	      or die "Cannot load Algorithm::Diff:\n";
+if ($diff_class) {
+    if ($diff_class =~ m/^(?:ad|pp|algorithm(?:-|::)diff(?:(?:-|::)pp)?)$/i) {
+	$diff_class =
+	    eval { require Algorithm::Diff;     "Algorithm::Diff";     }
+	}
+    elsif ($diff_class =~ m/^(?:adx|xs|algorithm(?:-|::)diff(?:-|::)xs)$/i) {
+	$diff_class = "Algorithm::Diff::XS";
+	$diff_class =
+	    eval { require Algorithm::Diff::XS; "Algorithm::Diff::XS"; }
+	}
+    else {
+	die "$diff_class is an unsupported Diff class\n";
+	}
+    }
+else {
+    $diff_class =
+	eval { require Algorithm::Diff::XS; "Algorithm::Diff::XS"; }
+     || eval { require Algorithm::Diff;     "Algorithm::Diff";     };
+    }
+$diff_class or die "Cannot load Algorithm::Diff:\n";
 
-caller or ccdiff (@ARGV);
+unless (caller) {
+    if (@ARGV == 2 && -d $ARGV[0] && -d $ARGV[1]) {
+	my ($d_from, $d_to) = @ARGV;
+	my %fn;
+	my @f;
+	foreach my $idx (0, 1) {
+	    my $dir = $ARGV[$idx];
+	    find (sub {
+		-f && -s or return;
+		my $fn = $File::Find::name =~ s{^$dir/*}{}r;
+		$fn{$fn}++;
+		$f[$idx]{$fn}++;
+		}, $dir);
+	    }
+	foreach my $fn (sort keys %fn) {
+	    if ($f[0]{$fn} && $f[1]{$fn}) {
+		ccdiff ("$d_from/$fn", "$d_to/$fn");
+		}
+	    elsif ($f[0]{$fn}) {
+		say "Only in $d_from: $fn";
+		}
+	    else {
+		say "Only in $d_to: $fn";
+		}
+	    delete $f[0]{$fn};
+	    }
+	}
+    else {
+	ccdiff (@ARGV);
+	}
+    exit 0;
+    }
 
 sub ccdiff {
     my $f1 = shift or usage (1);
@@ -265,7 +319,7 @@ sub ccdiff {
     -b $f1 || -c $f1 || -b $f2 || -c $f2 and
 	die "Character and block devices are not supported\n";
     -d $f1 || -d $f2 and
-	die "$CMD does not yet support recursive diff and/or directory diff\n";
+	die "$CMD does not support directory diff\n";
 
     my $fh;
 
@@ -545,7 +599,7 @@ sub read_rc {
     } # read_rc
 
 # Return the known colors from Term::ANSIColor
-# Stolen striaght from the pm
+# Stolen straight from the pm
 sub tac_colors {
     my %c256;
     foreach my $r (0 .. 5) {
@@ -594,6 +648,7 @@ ccdiff - Colored Character diff
 =head1 SYNOPSIS
 
  ccdiff [options] file1|- file2|-
+ ccdiff [options] dir1    dir2
 
  ccdiff --help
  ccdiff --man
@@ -601,6 +656,14 @@ ccdiff - Colored Character diff
 
 =head1 DESCRIPTION
 
+Show the diff between two files on a character by character base. In contrast to
+the standard diff tools, this tool uses the diff algorithm horizontally for each
+line in the vertical diff, highlighting the changes. This is very handy in hard
+to spot changes like C<O> to C<0>, C<I> to C<l> or C<1> and whitespace.
+
+If there are two argument, and both are a folder/directory, a recursive diff is
+executed. This is not available whan used as a (sub)class.
+
 =head1 OPTIONS
 
 =head2 Command line options
@@ -609,7 +672,7 @@ ccdiff - Colored Character diff
 
 =item --help -?
 
-Show a summary op the available command-line options and exit.
+Show a summary of the available command-line options and exit.
 
 =item --version -V
 
@@ -627,6 +690,28 @@ Show this manual using pod2text.
 
 All I/O (streams to compare and standard out) are in UTF-8.
 
+=item --diff-class=C --dc=C
+
+Select the class used to execute the diff. By default C<ccdiff> will select
+the first available out of C<Algorithm::Diff::XS> or C<Algorithm::Diff>.
+
+Sometime the C<XS> version fails on encoding and the pure-perl version will
+work just fine. You can force C<ccdiff> to use either
+
+Select the pure-perl version with any of C<PP>, C<AD>, C<Algorthm::Diff>,
+C<Algorithm-Diff>, or C<Algorithm::Diff::PP> (case insensitive)
+
+ --dc=pp
+ --dc=algorithm-diff
+ --diff-class=Algorithm::Diff::PP
+
+Select the XS version with any of C<XS>, C<ADX>, C<Algorthm::Diff::XS>, or
+C<Algorithm-Diff-XS> (case insensitive)
+
+ --dc=xs
+ --dc=algorithm-diff-xs
+ --diff-class=Algorithm::Diff::XS
+
 =item --unified[=3] -u [3]
 
 Generate a unified diff. The number of context lines is optional. When omitted
@@ -728,7 +813,7 @@ as C<chr_old> (the character used as marker under removed characters) and
 C<chr_new> (the character used as marker under added characters).
 
 If C<--ellipsis> is also in effect and either the C<chr_eli> is longer than
-one character or C<--verbose> level is over 2, this options is automatically
+one character or C<--verbose> level is over 2, this option is automatically
 disabled.
 
 =item --ascii -a
@@ -809,7 +894,7 @@ Defines the number of lines a change block may differ before the fall-back of
 horizontal diff to vertical diff.
 
 If a chunk describes a change, and the number of lines in the original block
-has less or more lines than the new block and that difference exceeds this
+has fewer or more lines than the new block and that difference exceeds this
 threshold, C<ccdiff> will fall-back to vertical diff.
 
 =item --heuristics=n -h n
@@ -860,12 +945,12 @@ and C<-B>.
 
 =item --ignore-trailing-space -Z
 
-Ignore changes in trailing white-space (TAB's and spaces).
+Ignore changes in trailing white-space (tabs and spaces).
 
 =item --ignore-ws|ignore-space-change -b
 
-Ignore changes in horizontal white-space (TAB's and spaces). This does not
-include white-space changes that splits non-white-space or removes white-space
+Ignore changes in horizontal white-space (tabs and spaces). This does not
+include white-space changes that split non-white-space or remove white-space
 between two non-white-space elements.
 
 =item --ignore-tab-expansion -E
@@ -891,10 +976,10 @@ in this order:
 and evaluated in that order. Any options specified in a file later in that
 chain will overwrite previously set options.
 
-Option files are only read and evaluated if it is not empty and not writable
+Option files are only read and evaluated if they are not empty and not writable
 by others than the owner.
 
-The syntax of the file is one option per line. where leading and trailing
+The syntax of the file is one option per line, where leading and trailing
 white-space is ignored. If that line then starts with one of the options
 listed below, followed by optional white-space followed by either an C<=> or
 a C<:>, followed by optional white-space and the values, the value is assigned
@@ -941,7 +1026,7 @@ default background color.
 
 =item swap (-s)
 
- reverse : false
+ swap    : false
 
 Swap the colors for new and old.
 
@@ -968,7 +1053,7 @@ This option may also be specified as
 
  old     : red
 
-Defines the color to be used for delete text. The default is C<red>.
+Defines the color to be used for deleted text. The default is C<red>.
 
 The color C<none> is also accepted and disables this color.
 
@@ -1015,7 +1100,7 @@ This option may also be specified as
 Defines if a header is displayed above the diff (default is 1), supported
 colors are allowed.
 
-If the values is a valid supported color, it will show the header in that
+If the value is a valid supported color, it will show the header in that
 color scheme.  To disable the header set it to C<0> in the RC file or use
 C<--no-header> as a command line argument.
 
@@ -1069,8 +1154,8 @@ with this option, it would be
  [002] 19,19d18
  [005] 42a42,42
 
-When this option contains a positive integer, C<ccdiff> will only show diff
-the diff chunk with that index.
+When this option contains a positive integer, C<ccdiff> will only show the
+diff chunk with that index.
 
 =item emacs
 
@@ -1141,7 +1226,7 @@ Defines the character used to indicate omitted text in large unchanged text
 when C<--ellipsis>/C<-e> is in effect.
 
 This character is not equally well visible on all terminals or in all fonts,
-so you might want to chane it to something that stands out better in you
+so you might want to change it to something that stands out better in your
 environment. Possible suggestions:
 
  … U+2026 HORIZONTAL ELLIPSIS
@@ -1170,7 +1255,7 @@ environment. Possible suggestions:
 
  chr_eli_v : U+21A4U+21A6
 
-When ussing C<--ellipsis> with C<--verbose> level 2 or up, the single character
+When using C<--ellipsis> with C<--verbose> level 2 or up, the single character
 indicator will be replaced with this character. If it is 2 characters wide, the
 length of the compressed part is put between the characters.
 
@@ -1186,7 +1271,7 @@ You can use ccdiff to show diffs in git. It may work like this:
  $ git config --global difftool.prompt false
  $ git config --global difftool.ccdiff.cmd 'ccdiff --utf-8 -u -r $LOCAL $REMOTE'
  $ git difftool SHA~..SHA
- $ wget https://github.com/Tux/App-ccdiff/blob/master/Files/git-ccdiff \
+ $ wget https://github.com/Tux/App-ccdiff/raw/master/Files/git-ccdiff \
     -O ~/bin/git-ccdiff
  $ perl -pi -e 's{/pro/bin/perl}{/usr/bin/env perl}' ~/bin/git-ccdiff
  $ chmod 755 ~/bin/git-ccdiff
@@ -1207,7 +1292,7 @@ kept in memory, this tool might not be able to deal with (very) large datasets.
 
 =head2 Speed
 
-There are situations where L<Algorithm::Diff> takes considerable more time
+There are situations where L<Algorithm::Diff> takes considerably more time
 compared to e.g. GNU diff. Installing L<Algorithm::Diff::XS> will make
 C<ccdiff> a lot faster. C<ccdiff> will choose L<Algorithm::Diff::XS> if
 available.
@@ -1222,7 +1307,7 @@ H.Merijn Brand
 
 =head1 COPYRIGHT AND LICENSE
 
- Copyright (C) 2018-2022 H.Merijn Brand.  All rights reserved.
+ Copyright (C) 2018-2023 H.Merijn Brand.  All rights reserved.
 
 This library is free software;  you can redistribute and/or modify it under
 the same terms as The Artistic License 2.0.
